@@ -19,13 +19,15 @@ from PIL import Image
 from rapidfuzz import fuzz
 
 from GameSentenceMiner import obs
-from GameSentenceMiner.util.electron_config import *
 from GameSentenceMiner.ocr.ss_picker import ScreenCropper
 from GameSentenceMiner.owocr.owocr.run import TextFiltering
 from GameSentenceMiner.util.configuration import get_config, get_app_directory, get_temporary_directory
 from GameSentenceMiner.ocr.gsm_ocr_config import OCRConfig, has_config_changed, set_dpi_awareness, get_window, get_ocr_config_path
-from GameSentenceMiner.owocr.owocr import screen_coordinate_picker, run
-from GameSentenceMiner.util.gsm_utils import sanitize_filename, do_text_replacements, OCR_REPLACEMENTS_FILE
+from GameSentenceMiner.owocr.owocr import run
+from GameSentenceMiner.util.electron_config import get_ocr_ocr2, get_ocr_send_to_clipboard, get_ocr_scan_rate, \
+    has_ocr_config_changed, reload_electron_config, get_ocr_two_pass_ocr, get_ocr_optimize_second_scan, \
+    get_ocr_language, get_ocr_manual_ocr_hotkey
+from GameSentenceMiner.util.gsm_utils import sanitize_filename
 import threading
 import time
 
@@ -414,23 +416,24 @@ done = False
 # Create a queue for tasks
 second_ocr_queue = queue.Queue()
 
-def get_ocr2_image(crop_coords, og_image, ocr2_engine=None):
+def get_ocr2_image(crop_coords, og_image: Image.Image, ocr2_engine=None):
     """
     Returns the image to use for the second OCR pass, cropping and scaling as needed.
     Logic is unchanged, but code is refactored for clarity and maintainability.
     """
     def return_original_image():
-        logger.info("Returning original image for OCR2 (no cropping or optimization).")
+        logger.debug("Returning original image for OCR2 (no cropping or optimization).")
         if not crop_coords or not get_ocr_optimize_second_scan():
             return og_image
         x1, y1, x2, y2 = crop_coords
-        x1 = max(0, min(x1, og_image.width))
-        y1 = max(0, min(y1, og_image.height))
-        x2 = max(x1, min(x2, og_image.width))
-        y2 = max(y1, min(y2, og_image.height))
+        x1 = min(max(0, x1), og_image.width)
+        y1 = min(max(0, y1), og_image.height)
+        x2 = min(max(0, x2), og_image.width)
+        y2 = min(max(0, y2), og_image.height)
         og_image.save(os.path.join(get_temporary_directory(), "pre_oneocrcrop.png"))
         return og_image.crop((x1, y1, x2, y2))
     
+    # TODO Get rid of this check, and just always convert to full res
     LOCAL_OCR_ENGINES = ['easyocr', 'oneocr', 'rapidocr', 'mangaocr', 'winrtocr']
     local_ocr = ocr2_engine in LOCAL_OCR_ENGINES
     ocr_config_local = copy(ocr_config)
@@ -444,10 +447,17 @@ def get_ocr2_image(crop_coords, og_image, ocr2_engine=None):
     obs_height = getattr(run.obs_screenshot_thread, 'height', None)
     if not obs_width or not obs_height:
         return return_original_image()
+    
     logger.debug(f"Getting OCR2 image with OBS dimensions: {obs_width}x{obs_height}")
 
     img = obs.get_screenshot_PIL(compression=100, img_format="jpg")
+    
     ocr_config_local.scale_to_custom_size(img.width, img.height)
+    
+    # If img.width and height is the same as obs, no need to scale coords, tolerance of .1%
+    if abs(img.width - obs_width) <= 0.1 * obs_width and abs(img.height - obs_height) <= 0.1 * obs_height:
+        logger.info("Image size matches OBS size, no need to scale coordinates.")
+        return return_original_image()
 
     # If no crop or optimization, just apply config and return
     if not crop_coords or not get_ocr_optimize_second_scan():
@@ -464,18 +474,19 @@ def get_ocr2_image(crop_coords, og_image, ocr2_engine=None):
     y1 = int(crop_coords[1] * height_ratio)
     x2 = int(crop_coords[2] * width_ratio)
     y2 = int(crop_coords[3] * height_ratio)
-    logger.debug(f"Scaled crop coordinates: {(x1, y1, x2, y2)}")
 
     # Clamp coordinates to image bounds
-    x1 = max(0, min(x1, img.width))
-    y1 = max(0, min(y1, img.height))
-    x2 = max(x1, min(x2, img.width))
-    y2 = max(y1, min(y2, img.height))
-
+    x1 = min(max(0, x1), img.width)
+    y1 = min(max(0, y1), img.height)
+    x2 = min(max(0, x2), img.width)
+    y2 = min(max(0, y2), img.height)
+    
+    logger.debug(f"Scaled crop coordinates: {(x1, y1, x2, y2)}")
+    
     img = run.apply_ocr_config_to_image(img, ocr_config_local, is_secondary=False)
     
-    
-    return img.crop((x1, y1, x2, y2))
+    ret = img.crop((x1, y1, x2, y2))
+    return ret
 
 def process_task_queue():
     while True:
@@ -537,9 +548,10 @@ def add_ss_hotkey(ss_hotkey="ctrl+shift+g"):
         ocr_config = get_ocr_config()
         img = obs.get_screenshot_PIL(compression=80, img_format="jpg")
         ocr_config.scale_to_custom_size(img.width, img.height)
-        img = run.apply_ocr_config_to_image(img, ocr_config, is_secondary=True)
-        do_second_ocr("", datetime.now(), img, TextFiltering(lang=get_ocr_language()), ignore_furigana_filter=True, ignore_previous_result=True)
-        
+        for rectangle in [rectangle for rectangle in ocr_config.rectangles if rectangle.is_secondary]:
+            new_img = run.apply_ocr_config_to_image(img, ocr_config, is_secondary=True, rectangles=[rectangle])
+            do_second_ocr("", datetime.now(), new_img, TextFiltering(lang=get_ocr_language()), ignore_furigana_filter=True, ignore_previous_result=True)
+
     filtering = TextFiltering(lang=get_ocr_language())
     cropper = ScreenCropper()
     def capture():
