@@ -106,6 +106,7 @@ def empty_post_process(text):
 
 def post_process(text, keep_blank_lines=False):
     import jaconv
+    text = text.replace("\"", "")
     if keep_blank_lines:
         text = '\n'.join([''.join(i.split()) for i in text.splitlines()])
     else:
@@ -354,8 +355,8 @@ class GoogleLens:
         response_proto = LensOverlayServerResponse().FromString(res.content)
         response_dict = response_proto.to_dict(betterproto.Casing.SNAKE)
 
-        if os.path.exists(r"C:\Users\Beangate\GSM\Electron App\test"):
-            with open(os.path.join(r"C:\Users\Beangate\GSM\Electron App\test", 'glens_response.json'), 'w', encoding='utf-8') as f:
+        if os.path.exists(r"C:\Users\Beangate\GSM\test"):
+            with open(os.path.join(r"C:\Users\Beangate\GSM\test", 'glens_response.json'), 'w', encoding='utf-8') as f:
                 json.dump(response_dict, f, indent=4, ensure_ascii=False)
         res = ''
         text = response_dict['objects_response']['text']
@@ -377,16 +378,13 @@ class GoogleLens:
                         res += 'BLANK_LINE'
                 for line in paragraph['lines']:
                     if furigana_filter_sensitivity:
+                        line_width = line['geometry']['bounding_box']['width'] * img.width
+                        line_height = line['geometry']['bounding_box']['height'] * img.height
                         for word in line['words']:
                             if self.punctuation_regex.findall(word['plain_text']):
                                 res += word['plain_text'] + word['text_separator']
                                 continue
-                            if 'geometry' not in word:
-                                res += word['plain_text'] + word['text_separator']
-                                continue
-                            word_width = word['geometry']['bounding_box']['width'] * img.width
-                            word_height = word['geometry']['bounding_box']['height'] * img.height
-                            if word_width > furigana_filter_sensitivity and word_height > furigana_filter_sensitivity:
+                            if line_width > furigana_filter_sensitivity and line_height > furigana_filter_sensitivity:
                                 res += word['plain_text'] + word['text_separator']
                             else:
                                 skipped.extend(word['plain_text'])
@@ -394,7 +392,8 @@ class GoogleLens:
                     else:
                         for word in line['words']:
                             res += word['plain_text'] + word['text_separator']
-                            
+                    res += '\n'
+
                 previous_line = paragraph
             res += '\n'
             # logger.info(
@@ -920,7 +919,7 @@ class OneOCR:
             self.regex = re.compile(
             r'[a-zA-Z\u00C0-\u00FF\u0100-\u017F\u0180-\u024F\u0250-\u02AF\u1D00-\u1D7F\u1D80-\u1DBF\u1E00-\u1EFF\u2C60-\u2C7F\uA720-\uA7FF\uAB30-\uAB6F]')
 
-    def __call__(self, img, furigana_filter_sensitivity=0, return_coords=False, multiple_crop_coords=False, return_one_box=True):
+    def __call__(self, img, furigana_filter_sensitivity=0, return_coords=False, multiple_crop_coords=False, return_one_box=True, return_dict=False):
         lang = get_ocr_language()
         if furigana_filter_sensitivity != None:
             furigana_filter_sensitivity = get_furigana_filter_sensitivity()
@@ -940,6 +939,7 @@ class OneOCR:
             return (False, 'Invalid image provided')
         crop_coords = None
         crop_coords_list = []
+        ocr_resp = ''
         if sys.platform == 'win32':
             try:
                 ocr_resp = self.model.recognize_pil(img)
@@ -959,17 +959,17 @@ class OneOCR:
                 boxes = []
                 if furigana_filter_sensitivity > 0:
                     for line in filtered_lines:
+                        line_x1, line_x2, line_x3, line_x4 = line['bounding_rect']['x1'], line['bounding_rect']['x2'], \
+                            line['bounding_rect']['x3'], line['bounding_rect']['x4']
+                        line_y1, line_y2, line_y3, line_y4 = line['bounding_rect']['y1'], line['bounding_rect']['y2'], \
+                            line['bounding_rect']['y3'], line['bounding_rect']['y4']
+                        line_width = max(line_x2 - line_x1, line_x3 - line_x4)
+                        line_height = max(line_y3 - line_y1, line_y4 - line_y2)
                         for char in line['words']:
                             if self.punctuation_regex.findall(char['text']):
                                 res += char['text']
                                 continue
-                            x1, x2, x3, x4 = char['bounding_rect']['x1'], char['bounding_rect']['x2'], \
-                            char['bounding_rect']['x3'], char['bounding_rect']['x4']
-                            y1, y2, y3, y4 = char['bounding_rect']['y1'], char['bounding_rect']['y2'], \
-                                char['bounding_rect']['y3'], char['bounding_rect']['y4']
-                            width = max(x2 - x1, x3 - x4)
-                            height = max(y3 - y1, y4 - y2)
-                            if width > furigana_filter_sensitivity and height > furigana_filter_sensitivity:
+                            if line_width > furigana_filter_sensitivity and line_height > furigana_filter_sensitivity:
                                 res += char['text']
                             else:
                                 skipped.extend(char for char in line['text'])
@@ -1014,7 +1014,6 @@ class OneOCR:
                     res = ocr_resp['text']
                     
                 if multiple_crop_coords:
-                    logger.info(f"Getting multiple crop coords for {len(filtered_lines)} lines")
                     for line in filtered_lines:
                         crop_coords_list.append(
                             (line['bounding_rect']['x1'] - 5, line['bounding_rect']['y1'] - 5,
@@ -1042,6 +1041,8 @@ class OneOCR:
             x.append(crop_coords_list)
         if return_one_box:
             x.append(crop_coords)
+        if return_dict:
+            x.append(ocr_resp)
         if is_path:
             img.close()
         return x
@@ -1242,7 +1243,7 @@ class OCRSpace:
 class GeminiOCR:
     name = 'gemini'
     readable_name = 'Gemini'
-    key = 'm'
+    key = ';'
     available = False
 
     def __init__(self, config={'api_key': None}, lang='ja'):
@@ -1432,13 +1433,14 @@ class localLLMOCR:
             self.keep_warm = config.get('keep_warm', True)
             self.custom_prompt = config.get('prompt', None)
             self.available = True
-            # if any(x in self.api_url for x in ['localhost', '127.0.0.1']):
-            #     if not self.check_connection(self.api_url):
-            #         logger.warning('Local LLM OCR API is not reachable')
-            #         return
+            if not self.check_url_for_connectivity(self.api_url):
+                self.available = False
+                logger.warning(f'Local LLM OCR API URL not reachable: {self.api_url}')
+                return
             self.client = openai.OpenAI(
                 base_url=self.api_url.replace('/v1/chat/completions', '/v1'),
-                api_key=self.api_key
+                api_key=self.api_key,
+                timeout=1
             )
             if self.client.models.retrieve(self.model):
                 self.model = self.model
@@ -1449,24 +1451,14 @@ class localLLMOCR:
         except Exception as e:
             logger.warning(f'Error initializing Local LLM OCR, Local LLM OCR will not work!')
             
-    def check_connection(self, url, port=None):
-        import http.client
-        conn = http.client.HTTPConnection(url, port or 1234, timeout=0.1)
+    def check_url_for_connectivity(self, url):
+        import requests
         try:
-            conn.request("GET", "/v1/models")
-            response = conn.getresponse()
-            if response.status == 200:
-                logger.info('Local LLM OCR API is reachable')
-                return True
-            else:
-                logger.warning('Local LLM OCR API is not reachable')
-                return False
-        except Exception as e:
-            logger.warning(f'Error connecting to Local LLM OCR API: {e}')
+            response = requests.get(url, timeout=0.5)
+            return response.status_code == 200
+        except Exception:
             return False
-        finally:
-            conn.close()
-        
+
     def keep_llm_warm(self):
         def ocr_blank_black_image():
             if self.last_ocr_time and (time.time() - self.last_ocr_time) < 5:
@@ -1492,7 +1484,7 @@ class localLLMOCR:
                 prompt = self.custom_prompt.strip()
             else:
                 prompt = f"""
-                Extract all {CommonLanguages.from_code(get_ocr_language()).name} Text from Image. Ignore all Furigana. Do not return any commentary, just the text in the image. If there is no text in the image, return "" (Empty String).
+                Extract all {CommonLanguages.from_code(get_ocr_language()).name} Text from Image. Ignore all Furigana. Do not return any commentary, just the text in the image. Do not Translate. If there is no text in the image, return "" (Empty String).
                 """
 
             response = self.client.chat.completions.create(
