@@ -1,10 +1,14 @@
+import json
+import logging
+import os
+import shutil
 import tempfile
 import time
 import warnings
 from abc import abstractmethod, ABC
 
 from GameSentenceMiner.util import configuration, ffmpeg
-from GameSentenceMiner.util.configuration import *
+from GameSentenceMiner.util.configuration import get_config, get_temporary_directory, logger, SILERO, WHISPER
 from GameSentenceMiner.util.ffmpeg import get_audio_length
 from GameSentenceMiner.util.gsm_utils import make_unique_file_name, run_new_thread
 from GameSentenceMiner.util.model import VADResult
@@ -37,19 +41,7 @@ class VADSystem:
             if not result.success and get_config().vad.backup_vad_model != configuration.OFF:
                 logger.info("No voice activity detected, using backup VAD model.")
                 result = self._do_vad_processing(get_config().vad.backup_vad_model, input_audio, output_audio, game_line)
-            if not result.success:
-                if get_config().vad.add_audio_on_no_results:
-                    logger.info("No voice activity detected, using full audio.")
-                    result.output_audio = input_audio
-                else:
-                    logger.info("No voice activity detected.")
-                    return result
-            else:
-                logger.info(result.trim_successful_string())
             return result
-        else:
-            return VADResult(True, 0, get_audio_length(input_audio), "OFF", [], input_audio)
-
 
     def _do_vad_processing(self, model, input_audio, output_audio, game_line):
         match model:
@@ -185,13 +177,14 @@ class WhisperVADProcessor(VADProcessor):
         # Transcribe the audio using Whisper
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            result: WhisperResult = self.vad_model.transcribe(temp_wav, vad=True, language=get_config().vad.language,
+            result: WhisperResult = self.vad_model.transcribe(temp_wav, vad=True, language=get_config().vad.language, vad_filter=get_config().vad.use_vad_filter_for_whisper,
                                                              temperature=0.0)
         voice_activity = []
 
-        logger.debug(json.dumps(result.to_dict(), indent=2))
+        logger.debug(json.dumps(result.to_dict()))
 
         # Process the segments to extract tokens, timestamps, and confidence
+        previous_segment = None
         for i, segment in enumerate(result.segments):
             if len(segment.text) <= 2 and ((i > 1 and segment.start - result.segments[i - 1].end > 1.0) or (i < len(result.segments) - 1 and result.segments[i + 1].start - segment.end > 1.0)):
                 if segment.text in ['えー', 'ん']:
@@ -201,10 +194,18 @@ class WhisperVADProcessor(VADProcessor):
                     logger.info(
                         "Unknown single character segment, not skipping, but logging, please report if this is a mistake: " + segment.text)
                     
-            if segment.no_speech_prob and segment.no_speech_prob > 0.6:
+            if segment.no_speech_prob and segment.no_speech_prob > 0.9:
                 logger.debug(f"Skipping segment with high no_speech_prob: {segment.no_speech_prob} for segment {segment.text} at {segment.start}-{segment.end}")
                 continue
-
+            
+            unique_words = set(word.word for word in segment.words)
+            if len(unique_words) <= 1 and len(segment.words) > 1:
+                logger.debug(f"Skipping segment with low unique words: {unique_words} for segment {segment.text} at {segment.start}-{segment.end}")
+                continue
+                
+            if segment.seek > 0 and segment.no_speech_prob > .3:
+                logger.debug(f"Skipping segment after long pause with high no_speech_prob after: {segment.no_speech_prob} for segment {segment.text} at {segment.start}-{segment.end}")
+                continue
 
             logger.debug(segment.to_dict())
             voice_activity.append({
@@ -225,15 +226,7 @@ class WhisperVADProcessor(VADProcessor):
             #             'confidence': word.probability
             #         })
 
-        # Analyze the detected words to decide whether to use the audio
-        should_use = False
-        unique_words = set(word['text'] for word in voice_activity)
-        if len(unique_words) > 1 or not all(item in ['えー', 'ん'] for item in unique_words):
-            should_use = True
-
-        if not should_use:
-            return None
-
+            previous_segment = segment
         # Return the detected voice activity and the total duration
         return voice_activity
 
@@ -405,6 +398,7 @@ def test_vad_processors():
     get_config().vad.cut_and_splice_segments = False
     get_config().vad.trim_beginning = True
     get_config().vad.add_audio_on_no_results = True
+    get_config().vad.use_vad_filter_for_whisper = False
     for processor, out_name in processors:
         logger.info("Testing Trim Audio with " + processor.vad_system_name)
         out_path = os.path.join(output_dir, out_name.replace("after_splice_", "after_trim_"))
